# Yet Another Pose Library

For Blender 3.3 - 4.3 (4.1 recommended).

During the Blender 3.x releases, a new Asset-based Pose system was introduced. That's fine and dandy, but it was not (and still is not as of Jan 2025) ready to replace the old Action-based Pose Library system. The former Pose Library was quickly deprecated and removed, leaving legacy users high and dry.

This addon brings the Legacy Pose Library back as best as it can in a modern panel, and tries to stick as close to the old system as possible. Most, if not all existing Pose Libraries should work without any modifications. Little new is introduced outside of UI enhancements and handling for new data.

These other excellent addons also address this removal in their own ways:
- [Sakura Poselib](https://github.com/kafuji/Sakura_Poselib)
- [gret](https://github.com/greisane/gret?tab=readme-ov-file#animation-actions-panel)
- [Amarillo's Pose Library](https://github.com/AmarilloArts/Amarillos-Pose-Library)

## Old Features

- The Pose Library list layout is ported 1:1 to a Sidebar (N) panel.

- Operators and data property [that were removed in 3.5](https://projects.blender.org/blender/blender/issues/93406) are ported from C to Python:
    - `yapl.apply_pose`
    - `yapl.browse_poses`
    - `yapl.create_pose_library`
    - `yapl.convert_pose_library`
    - `yapl.add_pose`
    - `yapl.move_pose`
    - `yapl.remove_pose`
    - `yapl.rename_pose`
    - `yapl.unlink_pose_library`
    - `Object.pose_library`

## New Features

- New (optional) panel interface for interacting with pose libraries, inspired by [gret's Actions Panel feature](https://github.com/greisane/gret?tab=readme-ov-file#animation-actions-panel).
    - Hotkey/Modifier-centric: Hover over buttons for tooltips

- Operator to protect potentially-orphaned pose data (read Considerations)
    - `yapl.protect_orphan_pose_library`

## Installation
1. [Download the repository as a zip](https://git.bkspl.me/breakingspell/YetAnotherPoseLibrary/archive/develop.zip), or otherwise clone the repository.
2. Install as an Add-on in Blender via Install -> Zip, and enable.
3. Optionally configure the suffix strings to fit your workflow, and whether the orhpan checker should run at startup.

## Usage
1. Open Sidebar (`N`), and choose the `Pose` tab.
2. Select existing Pose Library from drop-down, or create a new library.

### Hotkeys:
  - `Shift + L` - Add/Replace Pose
  - `Alt + L`   - Browse Poses with Arrow Keys
### Menu Controls
  - `Single click`  - Apply Pose
  - `Shift + Click` - Rename Pose
  - `Alt + Click`   - Remove Pose
  - `Ctrl + Click ` - Select Pose
  - Choose `Edit` for fast Move/Rename/Removal

## Considerations
- Opening older scenes will cause existing Pose Libraries to unlink from their former targets and fall into orphan state. They can be re-linked and will retain their link when saved afterwards, but would otherwise disappear if saved without linking or protecting the datablock.
  - This is due to the core DNA type `poselib` having been removed, so objects will drop the data. [There is no way to prevent this](https://developer.blender.org/docs/features/core/rna/#internals) as Python ID properties cannot be present at program runtime, only once the add-on is initialized.
  - To mitgate, an operator is provided to protect orphaned pose libraries: `yapl.protect_orphan_pose_library`.

- This add-on was made much easier by the `pose_markers` property being retained for converting old pose libraries to the new asset system. If they decide to remove that property as well, there will be a need to improvise an index-based lookup.
